<?php
/**
 * Created by PhpStorm.
 * User: matheussilva
 * Date: 18/08/14
 * Time: 15:34
 */

namespace Folha\Logic\ValesBeneficios\LancamentoValeTransporte;

use Folha\Entities\FpaTipoassoceve;
use Folha\Entities\FpaTipomoviment;
use \Folha\Entities\VabHistvalevinc;

use \Folha\Entities\FpaItecalcfolha;

use Folha\Repositories\FpaEventoh;
use HCM\Util\FPA;
use \Setforms2\Shared\Registry;

use \HCM\Util\DateUtil;

use \Geral\Util\NovoCodigo;

use \HCM\FPA\Services\LoteLancEventoService;

class Lancamento
{
    /**
     * Verifica se há algum evento 3030 e se encontrar, bloqueia a edição ou exclusão.
     * @param $nrcalcfolha
     * @param $nrvinculom
     * @param $dtmescompetencia
     * @param $nrorg
     * @throws \Exception
     */
    public static function verificaCalculo($nrcalcfolha, $nrvinculom, $dtmescompetencia, $nrorg)
    {
        try{
            $params = array(
                'P_NRCALCULOFOLHA'  => $nrcalcfolha,
                'P_NRVINCULOM'      => $nrvinculom,
                'P_NRORG'           => $nrorg);
            $where = "NRORG = :P_NRORG AND NRCALCULOFOLHA = :P_NRCALCULOFOLHA AND NRVINCULOM = :P_NRVINCULOM";
            $itemcalcs = FpaItecalcfolha::getObjectsByWhere( $where, $params);
            if($itemcalcs){
                foreach($itemcalcs as $itemcalc){
                    if($itemcalc->getNreventom() == FpaTipoassoceve::ASSOC_LIQUIDO){
                        throw new \Exception("Não foi possível fazer a modificação. Vinculo já possui calculo de folha na competencia.",30);
                    }
                }
            }

        }catch (\Exception $e){
            throw $e;
        }
    }

    /**
     * Gravar
     * -------------------------------------------------------------------------------------------------------
     * Action destinada à realização do cálculo do vale transporte e posterior gravação de um líquido
     * definitivo (evento 3030 Líquido) com o valor de vales cujos tipos estão definidos para gerar líquido.
     * Neste caso, a manipulação manual sobre o histórico de vale transporte para o respectivo vínculo não
     * será mais permitida.
     * --------------------------------------------------------------------------------------------------------
     * @param int $nrcalcfolha
     * @param string $dtmescompetencia
     * @param int $nrorg
     */
    public static function grava($nrcalcfolha, $dtmescompetencia, $nrorg, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom = 'A', $tipodelinha = null, $dtpgtovale, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao)
    {
        // -------------------------------------------- TRECHO DE ROTINA NÃO NECESSÁRIO. NÃO EXPORTARÁ DIRETAMENTE P/ FOLHA, MAS GERARÁ LOTE.
        /* 

        $nrmovimentomensal = FpaTipomoviment::TP_MOV_MENSAL;

        $nrcalculomensal = FPA::determinaNrCalculoFolha($nrmovimentomensal);

        if($tipodelinha) {
            $wheretipolinha = " AND NRVINCULOM IN
                                (SELECT NRVINCULOM
                                FROM VAB_HISTVALEVINC
                                WHERE VAB_HISTVALEVINC.NRORG         = :P_NRORG
                                AND VAB_HISTVALEVINC.NRTIPOLINHAVINC = ".$tipodelinha."
                                AND e.NRVINCULOM      = VAB_HISTVALEVINC.NRVINCULOM
                                AND VAB_HISTVALEVINC.DTMESCOMPAPVALE = TO_DATE(:P_DTOCORRENCIA, 'DD/MM/YYYY')
                                )";
        } else {
            $wheretipolinha = null;
        }

        $params = array(
            'P_NREVENTOM'        => Fpatipoassoceve::ASSOC_VALE_TRANSP_RECEB,
            'P_NRCALCULOFOLHA'   => $nrcalculomensal,
            'P_DTOCORRENCIA'     => $dtmescompetencia,
            'P_NRORG'            => $nrorg,
            'P_NRESTRUTSIND'     => $nrestrutsind,
            'P_NRESTRUTLEGAL'    => $nrestrutlegal,
            'P_NRESTRUTGEREN'    => $nrestrutgeren,
            'P_LISTA_NRVINCULOM' => $lista_nrvinculom
        );
        $where = "NREVENTOM = :P_NREVENTOM 
                 AND NRCALCULOFOLHA = :P_NRCALCULOFOLHA 
                 AND NRORG = :P_NRORG 
                 AND (INSTR(:P_LISTA_NRVINCULOM, '_'||NRVINCULOM||'_') > 0 OR :P_LISTA_NRVINCULOM = 'A')
                 ".$wheretipolinha;
        $itemcalcs = FpaItecalcfolha::getObjectsByWhere($where, $params);

        if($itemcalcs){
            throw new \Exception("Não foi possível calcular. Existe(m) vínculo(s) exportado(s) para a folha.",30);
        }
        */ //--------------------------------------------
        self::insereIteCalcFolha(3030, $nrcalcfolha, $dtmescompetencia, $nrorg, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $tipodelinha, $dtpgtovale, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao);
    }

    /**
     * Gravar Cálculo Parcial
     * ----------------------------------------------------------------------------------------------------
     * Action destinada à realização do cálculo do vale transporte e posterior gravação de um líquido
     * parcial (evento 3040 Líquido Parcial) com o valor de vales cujos tipos estão definidos para gerar
     * líquido.
     * ----------------------------------------------------------------------------------------------------
     * @param int $nrcalcfolha
     * @param string $dtmescompetencia
     * @param int $nrorg
     */
    public static function gravaParcial($nrcalcfolha, $dtmescompetencia, $nrorg, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom = 'A', $tipodelinha = null, $dtpgtovale, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao)
    {
        
        // -------------------------------------------- TRECHO DE ROTINA NÃO NECESSÁRIO. NÃO EXPORTARÁ DIRETAMENTE P/ FOLHA, MAS GERARÁ LOTE.
        /* 
        $nrmovimentomensal = FpaTipomoviment::TP_MOV_MENSAL;

        $nrcalculomensal = FPA::determinaNrCalculoFolha($nrmovimentomensal);

        if($tipodelinha) {
            $wheretipolinha = " AND NRVINCULOM IN
                                (SELECT NRVINCULOM
                                FROM VAB_HISTVALEVINC
                                WHERE VAB_HISTVALEVINC.NRORG         = :P_NRORG
                                AND VAB_HISTVALEVINC.NRTIPOLINHAVINC = ".$tipodelinha."
                                AND e.NRVINCULOM      = VAB_HISTVALEVINC.NRVINCULOM
                                AND VAB_HISTVALEVINC.DTMESCOMPAPVALE = TO_DATE(:P_DTOCORRENCIA, 'DD/MM/YYYY')
                                )";
        } else {
            $wheretipolinha = null;
        }

        $params = array(
            'P_NREVENTOM'       => Fpatipoassoceve::ASSOC_VALE_TRANSP_RECEB,
            'P_NRCALCULOFOLHA'  => $nrcalculomensal,
            'P_DTOCORRENCIA'    => $dtmescompetencia,
            'P_NRORG'           => $nrorg,
            'P_LISTA_NRVINCULOM' => $lista_nrvinculom
        );
        $where = "NREVENTOM = :P_NREVENTOM 
                  AND NRCALCULOFOLHA = :P_NRCALCULOFOLHA 
                  AND NRORG = :P_NRORG 
                  AND (INSTR(:P_LISTA_NRVINCULOM, '_'||NRVINCULOM||'_') > 0 OR :P_LISTA_NRVINCULOM = 'A')
                  ".$wheretipolinha;
        $itemcalcs = FpaItecalcfolha::getObjectsByWhere($where, $params);

        if($itemcalcs){
            throw new \Exception("Não foi possível calcular. Existe(m) vínculo(s) exportado(s) para a folha.",30);
        }
        */ //--------------------------------------------

        $condEstrutura = $wheretipolinha = "";
        $condAdmissao = "";
        $condEstrutura = Lancamento::buildWhereEstrutura($nrestrutsind, $nrestrutlegal, $nrestrutgeren);
        $condAdmissao = Lancamento::buildWhereDtAdmissao($dtiniadmissao, $dtfimadmissao);
        $wheretipolinha = Lancamento::buildWhereTpLinha($tipodelinha);
        
        $params = array(
            'P_NREVENTOM'       => Fpatipoassoceve::ASSOC_LIQUIDO,
            'P_DTMESCOMPETENC' => $dtmescompetencia,
            'P_NRCALCULOFOLHA'   => $nrcalcfolha,
            'P_DTOCORRENCIA'    => $dtmescompetencia,
            'P_NRORG'            => $nrorg,
            'P_NRESTRUTSIND'     => $nrestrutsind,
            'P_NRESTRUTLEGAL'    => $nrestrutlegal,
            'P_NRESTRUTGEREN'    => $nrestrutgeren,
            'P_LISTA_NRVINCULOM' => $lista_nrvinculom,
            'P_LISTA_LOTE_VINCULOS' => $lista_lotevinculos,
            'P_DTINIADMISSAO' => $dtiniadmissao,
            'P_DTFIMADMISSAO' => $dtfimadmissao,
            'P_NRESTRUTLEGALOPER'    => FPA::getNrestrutlegalOper(),
            'P_NRESTRUTGERENOPER'    => FPA::getNrestrutgerenOper()
        );
        $where = "NREVENTOM = :P_NREVENTOM 
                  AND NRCALCULOFOLHA = :P_NRCALCULOFOLHA 
                  AND NRORG = :P_NRORG 
                  AND (INSTR(:P_LISTA_NRVINCULOM, '_'||NRVINCULOM||'_') > 0 OR :P_LISTA_NRVINCULOM = 'A')"
                  .$wheretipolinha
                  .$condEstrutura
                  .$condAdmissao;
        $itemcalcs = FpaItecalcfolha::getObjectsByWhere($where, $params);
        
        if($itemcalcs){
            throw new \Exception("Não foi possível calcular. Vínculo(s) possui(em) líquido.",30);
        }
        
        self::insereIteCalcFolha(3040, $nrcalcfolha, $dtmescompetencia, $nrorg, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $tipodelinha, $dtpgtovale, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao);
    }

    /**
     * Retroceder Total
     * ------------------------------------------------------------------------------------------------------
     * Action destinada à exclusão dos itens de cálculo do movimento de vale transporte e o histórico de
     * recebimento do vale transporte conforme filtro preestabelecido na tela de histórico. Neste caso, o
     * cálculo do vale transporte terá que ser refeito.
     * ------------------------------------------------------------------------------------------------------
     * @param int $nrcalcfolha
     * @param string $dtmescompetencia
     * @param int $nrorg
     * @param mixed $lista_nrvinculom
     */
    public static function retrocede($nrcalcfolha, $dtmescompetencia, $nrorg, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom = 'A', $tipodelinha = null, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao)
    {
        $nrmovimentovale = Fpatipomoviment::TP_MOV_VALE_TRANSPORTE;
        $nrmovimentomensal = Fpatipomoviment::TP_MOV_MENSAL;

        $nrcalculomensal = FPA::determinaNrCalculoFolha($nrmovimentomensal);

        $condEstrutura = $wheretipolinha = "";
        $condAdmissao = "";
        $condAdmissao = Lancamento::buildWhereDtAdmissao($dtiniadmissao, $dtfimadmissao);
        $condEstrutura = Lancamento::buildWhereEstrutura($nrestrutsind, $nrestrutlegal, $nrestrutgeren);
        $wheretipolinha = Lancamento::buildWhereTpLinha($tipodelinha);


        $params = array(
            'P_NREVENTOM'       => Fpatipoassoceve::ASSOC_VALE_TRANSP_RECEB,
            'P_NRCALCULOFOLHA'  => $nrcalculomensal,
            'P_DTMESCOMPETENC' => $dtmescompetencia,
            'P_DTOCORRENCIA'    => $dtmescompetencia,
            'P_NRORG'           => $nrorg,
            'P_NRESTRUTSIND'     => $nrestrutsind,
            'P_NRESTRUTLEGAL'    => $nrestrutlegal,
            'P_NRESTRUTGEREN'    => $nrestrutgeren,
            'P_LISTA_NRVINCULOM' => $lista_nrvinculom,
            'P_LISTA_LOTE_VINCULOS' => $lista_lotevinculos,
            'P_DTINIADMISSAO' => $dtiniadmissao,
            'P_DTFIMADMISSAO' => $dtfimadmissao,
            'P_NRESTRUTLEGALOPER'    => FPA::getNrestrutlegalOper(),
            'P_NRESTRUTGERENOPER'    => FPA::getNrestrutgerenOper()
        );
        $where = "NREVENTOM = :P_NREVENTOM 
                 AND NRCALCULOFOLHA = :P_NRCALCULOFOLHA 
                 AND NRORG = :P_NRORG 
                 AND (INSTR(:P_LISTA_NRVINCULOM, '_'||NRVINCULOM||'_') > 0 OR :P_LISTA_NRVINCULOM = 'A')"
                 .$wheretipolinha
                 .$condEstrutura
                 .$condAdmissao;
        $itemcalcs = FpaItecalcfolha::getObjectsByWhere($where, $params);

        if($itemcalcs){
            throw new \Exception("Não foi possível retroceder. Existe(m) vínculo(s) exportado(s) para a folha.",30);
        }

        //$where = "NREVENTOM <> 3030 AND NRCALCULOFOLHA = :P_NRCALCULOFOLHA AND NRORG = :P_NRORG AND  TO_CHAR(DTOCORRENCIA, 'DD/MM/YYYY') = '$dtmescompetencia'";
        $params = array(
            'P_NRORG'           => $nrorg,
            'P_NRCALCULOFOLHA'  => $nrcalcfolha,
            'P_DTMESCOMPETENC' => $dtmescompetencia,
            'P_DTOCORRENCIA'    => $dtmescompetencia,
            'P_NRESTRUTSIND'     => $nrestrutsind,
            'P_NRESTRUTLEGAL'    => $nrestrutlegal,
            'P_NRESTRUTGEREN'    => $nrestrutgeren,
            'P_LISTA_NRVINCULOM'=> $lista_nrvinculom,
            'P_NRCALCULOMENSAL' => $nrcalculomensal,
            'P_NRTIPOMOVIMENTVALE' => $nrmovimentovale,
            'P_LISTA_LOTE_VINCULOS' => $lista_lotevinculos,
            'P_DTINIADMISSAO' => $dtiniadmissao,
            'P_DTFIMADMISSAO' => $dtfimadmissao,
            'P_NRESTRUTLEGALOPER'    => FPA::getNrestrutlegalOper(),
            'P_NRESTRUTGERENOPER'    => FPA::getNrestrutgerenOper()
        );

        $where = "NRORG = :P_NRORG 
                  AND NRCALCULOFOLHA  = (SELECT NRCALCULOFOLHA FROM FPA_CALCULOFOLHA WHERE NRCALCULOFOLHA = :P_NRCALCULOFOLHA AND NRORG = :P_NRORG AND NRTIPOMOVIMENT = :P_NRTIPOMOVIMENTVALE) 
                  AND :P_DTOCORRENCIA = :P_DTOCORRENCIA   
                  AND (NREVENTOM IN (3040, 480, 3030))    
                  AND NRVINCULOM NOT IN (SELECT LIQ_ITECALCFOLHA.NRVINCULOM FROM FPA_ITECALCFOLHA LIQ_ITECALCFOLHA WHERE LIQ_ITECALCFOLHA.NRORG = :P_NRORG AND LIQ_ITECALCFOLHA.NRCALCULOFOLHA = :P_NRCALCULOMENSAL AND LIQ_ITECALCFOLHA.NREVENTOM = 3030) 
                  AND (INSTR(:P_LISTA_NRVINCULOM, '_'||NRVINCULOM||'_') > 0 OR :P_LISTA_NRVINCULOM = 'A')"
                  .$wheretipolinha
                  .$condEstrutura
                  .$condAdmissao;
        $itemcalcs = FpaItecalcfolha::getObjectsByWhere($where, $params);

        /**
         * @var FpaItecalcfolha $itemcalc
         */
        foreach($itemcalcs as $itemcalc){
            $itemcalc->delete();
        }

        $params = array(
            'P_NRORG'           => $nrorg,
            'P_NRCALCULOFOLHA'  => $nrcalcfolha,
            'P_DTMESCOMPETENC' => $dtmescompetencia,
            'P_DTOCORRENCIA'    => $dtmescompetencia,
            'P_NRESTRUTSIND'     => $nrestrutsind,
            'P_NRESTRUTLEGAL'    => $nrestrutlegal,
            'P_NRESTRUTGEREN'    => $nrestrutgeren,
            'P_LISTA_NRVINCULOM'=> $lista_nrvinculom,
            'P_NRCALCULOMENSAL' => $nrcalculomensal,
            'P_NRTIPOMOVIMENTVALE' => $nrmovimentovale,
            'P_LISTA_LOTE_VINCULOS' => $lista_lotevinculos,
            'P_DTINIADMISSAO' => $dtiniadmissao,
            'P_DTFIMADMISSAO' => $dtfimadmissao
        );

        if($tipodelinha) {
            $wheretipolinha = " AND NRTIPOLINHAVINC = ".$tipodelinha;
        } else {
            $wheretipolinha = null;
        }

        $where = "NRORG = :P_NRORG 
                  AND :P_DTOCORRENCIA = :P_DTOCORRENCIA	  
                  AND NRCALCULOFOLHA  = (SELECT NRCALCULOFOLHA FROM FPA_CALCULOFOLHA WHERE NRCALCULOFOLHA = :P_NRCALCULOFOLHA AND NRORG = :P_NRORG AND NRTIPOMOVIMENT = :P_NRTIPOMOVIMENTVALE) 
                  AND NRVINCULOM NOT IN (SELECT LIQ_ITECALCFOLHA.NRVINCULOM FROM FPA_ITECALCFOLHA LIQ_ITECALCFOLHA WHERE LIQ_ITECALCFOLHA.NRORG = :P_NRORG AND LIQ_ITECALCFOLHA.NRCALCULOFOLHA = :P_NRCALCULOMENSAL AND LIQ_ITECALCFOLHA.NREVENTOM = 3030) 
                  AND (INSTR(:P_LISTA_NRVINCULOM, '_'||NRVINCULOM||'_') > 0 OR :P_LISTA_NRVINCULOM = 'A')"
                 .$wheretipolinha
                 .$condEstrutura
                 .$condAdmissao;
        $hists = VabHistvalevinc::getObjectsByWhere( $where, $params);
        foreach($hists as $hist){
            $hist->delete();
        }


        Registry::getEntityManager()->flush();
    }

    /**
     * Retroceder Somente Movimento
     * ------------------------------------------------------------------------------------------------------
     * Action destinada à exclusão dos itens de cálculo do movimento de vale transporte conforme filtro
     * preestabelecido na tela de histórico. Desta forma, o operador poderá realizar a manutenção do
     * histórico dos respectivos vínculos antes de realizar uma nova gravação do mesmo.
     * ------------------------------------------------------------------------------------------------------
     * @param int $nrcalcfolha
     * @param string $dtmescompetencia
     * @param int $nrorg
     * @param mixed $lista_nrvinculom
     */
    public static function retrocedeParcial($nrcalcfolha, $dtmescompetencia, $nrorg, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom = 'A', $tipodelinha = null, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao)
    {
        $nrmovimentovale = Fpatipomoviment::TP_MOV_VALE_TRANSPORTE;
        $nrmovimentomensal = Fpatipomoviment::TP_MOV_MENSAL;
        $nrcalculomensal = FPA::determinaNrCalculoFolha($nrmovimentomensal);

        $condEstrutura = $wheretipolinha = "";
        $condAdmissao = "";
        
        $condAdmissao = Lancamento::buildWhereDtAdmissao($dtiniadmissao, $dtfimadmissao);
        $condEstrutura = Lancamento::buildWhereEstrutura($nrestrutsind, $nrestrutlegal, $nrestrutgeren);
        $wheretipolinha = Lancamento::buildWhereTpLinha($tipodelinha);

        $params = array(
            'P_NREVENTOM'        => Fpatipoassoceve::ASSOC_VALE_TRANSP_RECEB,
            'P_NRCALCULOFOLHA'   => $nrcalculomensal,
            'P_DTOCORRENCIA'     => $dtmescompetencia,
            'P_DTMESCOMPETENC' => $dtmescompetencia,
            'P_NRORG'            => $nrorg,
            'P_NRESTRUTSIND'     => $nrestrutsind,
            'P_NRESTRUTLEGAL'    => $nrestrutlegal,
            'P_NRESTRUTGEREN'    => $nrestrutgeren,
            'P_LISTA_NRVINCULOM' => $lista_nrvinculom,
            'P_LISTA_LOTE_VINCULOS' => $lista_lotevinculos,
            'P_DTINIADMISSAO' => $dtiniadmissao,
            'P_DTFIMADMISSAO' => $dtfimadmissao,
            'P_NRESTRUTLEGALOPER'    => FPA::getNrestrutlegalOper(),
            'P_NRESTRUTGERENOPER'    => FPA::getNrestrutgerenOper()
        );

        $where = "NREVENTOM = :P_NREVENTOM 
                  AND NRCALCULOFOLHA = :P_NRCALCULOFOLHA 
                  AND NRORG = :P_NRORG 
                  AND ( INSTR(:P_LISTA_NRVINCULOM, '_'||NRVINCULOM||'_') > 0 OR :P_LISTA_NRVINCULOM = 'A' )"
                  .$wheretipolinha
                  .$condEstrutura
                  .$condAdmissao;
        $itemcalcs = FpaItecalcfolha::getObjectsByWhere($where, $params);
        if($itemcalcs){
            throw new \Exception("Não foi possível retroceder. Existe(m) vínculo(s) exportado(s) para a folha.",30);
        }

        $params = array(
            'P_NRORG'           => $nrorg,
            'P_NRCALCULOFOLHA'  => $nrcalcfolha,
            'P_DTMESCOMPETENC' => $dtmescompetencia,
            'P_DTOCORRENCIA'    => $dtmescompetencia,
            'P_NRESTRUTSIND'     => $nrestrutsind,
            'P_NRESTRUTLEGAL'    => $nrestrutlegal,
            'P_NRESTRUTGEREN'    => $nrestrutgeren,
            'P_LISTA_NRVINCULOM'   => $lista_nrvinculom,
            'P_NRCALCULOMENSAL'    => $nrcalculomensal,
            'P_NRTIPOMOVIMENTVALE' => $nrmovimentovale,
            'P_LISTA_LOTE_VINCULOS' => $lista_lotevinculos,
            'P_DTINIADMISSAO' => $dtiniadmissao,
            'P_DTFIMADMISSAO' => $dtfimadmissao,
            'P_NRESTRUTLEGALOPER'    => FPA::getNrestrutlegalOper(),
            'P_NRESTRUTGERENOPER'    => FPA::getNrestrutgerenOper()
        );
    
        $where = "NRORG = :P_NRORG 
                  AND NRCALCULOFOLHA = (SELECT NRCALCULOFOLHA FROM FPA_CALCULOFOLHA WHERE NRCALCULOFOLHA = :P_NRCALCULOFOLHA AND NRORG = :P_NRORG AND NRTIPOMOVIMENT = :P_NRTIPOMOVIMENTVALE) 
                  AND NREVENTOM IN (3030, 3040, 480) 
                  AND NRVINCULOM NOT IN (SELECT LIQ_ITECALCFOLHA.NRVINCULOM FROM FPA_ITECALCFOLHA LIQ_ITECALCFOLHA WHERE LIQ_ITECALCFOLHA.NRORG = :P_NRORG AND LIQ_ITECALCFOLHA.NRCALCULOFOLHA = :P_NRCALCULOMENSAL AND LIQ_ITECALCFOLHA.NREVENTOM = 3030) 
                  AND (INSTR(:P_LISTA_NRVINCULOM, '_'||NRVINCULOM||'_') > 0 OR :P_LISTA_NRVINCULOM = 'A')"
                  .$wheretipolinha
                  .$condEstrutura
                  .$condAdmissao;
        $itemcalcs = FpaItecalcfolha::getObjectsByWhere($where, $params);
        foreach($itemcalcs as $itemcalc){
            $itemcalc->delete();
        }

        Registry::getEntityManager()->flush();

    }

    
    /**
     * Exportar para Folha
     * ------------------------------------------------------------------------------------------------------
     * Action destinada à exportação dos registros do evento “480 Vale Trans Recebido” para o movimento
     * mensal da respectiva competência e, consequentemente, possibilitar que o cálculo da folha mensal
     * possa gerar o desconto de vale transporte para o respectivo vínculo.
     * Esta opção será permitida somente se o movimento mensal já não possuir um líquido definitivo para
     * o respectivo vínculo e, além disso, o acionamento desta opção será permitido somente após a
     * gravação do histórico de vale transporte (garantindo que o valor do evento “480 Vale Transp
     * Recebido” seja igual no movimento exclusivo para o vale transporte e o exportado para o
     * movimento principal da folha mensal).
     * ------------------------------------------------------------------------------------------------------
     */
    public static function exportarFolha($nrcalculofolhavale, $dtmescompetencia, $cdoperador, $nrorg, $nrtpmovimento = 1, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom = 'A')
    {
        try {
            $nrvinculoms = array();
            $nrcalculofolha = FPA::determinaNrCalculoFolha($nrtpmovimento);
            $itemmovcalcs = \Folha\Repositories\FpaItecalcfolha::getVinculosFolha($nrcalculofolhavale, $dtmescompetencia, $nrorg, $lista_nrvinculom);

            if (!$itemmovcalcs) {
                throw new \Exception("Não foi encontrado movimento de vale.",30);
            }

            $params = array(
                'P_NRORG'           => $nrorg,
                'P_NRCALCULOFOLHA'  => $nrcalculofolhavale,
                'P_DTOCORRENCIA'    => $dtmescompetencia,
                'P_LISTA_NRVINCULOM'=> $lista_nrvinculom,
                'P_NRTIPOMOVIMENTVALE' => Fpatipomoviment::TP_MOV_VALE_TRANSPORTE,
            );
            $where = "NRORG = :P_NRORG AND
                  NRCALCULOFOLHA = (SELECT NRCALCULOFOLHA FROM FPA_CALCULOFOLHA WHERE NRCALCULOFOLHA = :P_NRCALCULOFOLHA AND NRORG = :P_NRORG AND NRTIPOMOVIMENT = :P_NRTIPOMOVIMENTVALE) AND
                  NREVENTOM = 3040 AND
                  (INSTR(:P_LISTA_NRVINCULOM, '_'||NRVINCULOM||'_') > 0 OR :P_LISTA_NRVINCULOM = 'A')";
            $itemcalcs = FpaItecalcfolha::getObjectsByWhere($where, $params);

            if ($itemcalcs) {
                throw new \Exception("Vínculo(s) com parcial calculado. Favor calcular total para exportar para a folha.",20);
            }

            $nreventom = Fpatipoassoceve::ASSOC_VALE_TRANSP_RECEB;
            $ideventodemonst = FpaEventoh::getIdeventodemonst($nreventom, $dtmescompetencia);

            foreach ($itemmovcalcs as $itemmovcalc) {
                self::insereBancoIteCalcFolha($ideventodemonst, $nreventom, $nrcalculofolha, $dtmescompetencia, $nrorg, $itemmovcalc);
                $nrvinculoms[] = $itemmovcalc['NRVINCULOM'];
            }
            Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Insere um item no cálculo folha para cada vínculo
     * @param int $nrevento
     * @param int $nrcalcfolha
     * @param string $dtmescompetencia
     * @param int $nrorg
     * @param mixed $lista_nrvinculom
     */
    public static function insereIteCalcFolha($nrevento, $nrcalcfolha, $dtmescompetencia, $nrorg, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom = 'A', $tipodelinha = null, $dtpgtovale, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao)
    {
        /**
         * @var FpaItecalcfolha $itemcalc
         */
        $condEstrutura = $wheretipolinha = "";
        $condAdmissao = "";
        $condEstrutura = Lancamento::buildWhereEstrutura($nrestrutsind, $nrestrutlegal, $nrestrutgeren);
        $condAdmissao = Lancamento::buildWhereDtAdmissao($dtiniadmissao, $dtfimadmissao);
        $wheretipolinha = Lancamento::buildWhereTpLinha($tipodelinha);
        
        $nrlotevinc = !empty($lista_lotevinculos) ? $lista_lotevinculos : null;
        
        //Deleta todos os itens cálculo vínculo de eventos 3040 e 480
        $nrmovimentovale = Fpatipomoviment::TP_MOV_VALE_TRANSPORTE;
        $params = array(
            'P_NRORG'           => $nrorg,
            'P_DTMESCOMPETENC' => $dtmescompetencia,
            'P_NRCALCULOFOLHA'  => $nrcalcfolha,
            'P_DTOCORRENCIA'    => $dtmescompetencia,
            'P_NRESTRUTSIND'     => $nrestrutsind,
            'P_NRESTRUTLEGAL'    => $nrestrutlegal,
            'P_NRESTRUTGEREN'    => $nrestrutgeren,
            'P_LISTA_NRVINCULOM'=> $lista_nrvinculom,
            'P_NRTIPOMOVIMENTVALE' => $nrmovimentovale,
            'P_LISTA_LOTE_VINCULOS' => $lista_lotevinculos,
            'P_DTINIADMISSAO' => $dtiniadmissao,
            'P_DTFIMADMISSAO' => $dtfimadmissao,
            'P_NRESTRUTLEGALOPER'    => FPA::getNrestrutlegalOper(),
            'P_NRESTRUTGERENOPER'    => FPA::getNrestrutgerenOper()
        );

        $where = "NRORG = :P_NRORG 
                  AND NRCALCULOFOLHA = (SELECT NRCALCULOFOLHA FROM FPA_CALCULOFOLHA WHERE NRCALCULOFOLHA = :P_NRCALCULOFOLHA AND NRORG = :P_NRORG AND NRTIPOMOVIMENT = :P_NRTIPOMOVIMENTVALE) 
                  AND NREVENTOM IN (3040, 480) 
                  AND NRVINCULOM NOT IN (SELECT LIQ_ITECALCFOLHA.NRVINCULOM FROM FPA_ITECALCFOLHA LIQ_ITECALCFOLHA WHERE LIQ_ITECALCFOLHA.NRORG = :P_NRORG AND LIQ_ITECALCFOLHA.NRCALCULOFOLHA = :P_NRCALCULOFOLHA  AND LIQ_ITECALCFOLHA.NREVENTOM = 3030) 
                  AND (INSTR(:P_LISTA_NRVINCULOM, '_'||NRVINCULOM||'_') > 0 OR :P_LISTA_NRVINCULOM = 'A')"
                  .$wheretipolinha
                  .$condEstrutura
                  .$condAdmissao;
        $itemcalcs = FpaItecalcfolha::getObjectsByWhere($where, $params);
        foreach($itemcalcs as $itemcalc){
            $itemcalc->delete();
        }

        //Registry::getEntityManager()->flush();

        $params = array(
            'P_NRORG'           => $nrorg,
            'P_DTMESCOMPETENC' => $dtmescompetencia,
            'P_NRCALCULOFOLHA'  => $nrcalcfolha,
            'P_DTOCORRENCIA'    => $dtpgtovale,
            'P_NRESTRUTSIND'     => $nrestrutsind,
            'P_NRESTRUTLEGAL'    => $nrestrutlegal,
            'P_NRESTRUTGEREN'    => $nrestrutgeren,
            "P_LISTA_NRVINCULOM" => $lista_nrvinculom,
            'P_LISTA_LOTE_VINCULOS' => $lista_lotevinculos,
            'P_DTINIADMISSAO' => $dtiniadmissao,
            'P_DTFIMADMISSAO' => $dtfimadmissao,
            'P_NRESTRUTLEGALOPER'    => FPA::getNrestrutlegalOper(),
            'P_NRESTRUTGERENOPER'    => FPA::getNrestrutgerenOper()
        );
        $where = "NREVENTOM = 3030 
                  AND NRCALCULOFOLHA = :P_NRCALCULOFOLHA 
                  AND NRORG = :P_NRORG 
                  AND DTOCORRENCIA = :P_DTOCORRENCIA "
                  .$wheretipolinha
                  .$condEstrutura
                  .$condAdmissao;
        
        $itemcalcs = FpaItecalcfolha::getObjectsByWhere($where, $params);
        $itens = array(0);
        foreach($itemcalcs as $itemcalc){
            $itens[] = $itemcalc->getNrvinculom();
        }
        $itens = implode(', ', $itens);

        Registry::getEntityManager()->flush();
        $liquidos = \Folha\Repositories\FpaItecalcfolha::getVinculosLiquidoVale($nrcalcfolha, $dtmescompetencia, $nrorg, $itens, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom,$tipodelinha, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao);
        $vales    = \Folha\Repositories\FpaItecalcfolha::getVinculosVale($nrcalcfolha, $dtmescompetencia, $nrorg, $itens, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $tipodelinha, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao);
        
        if(!empty($liquidos)) {
            foreach($liquidos as $liquido){
                $ideventodemonst = FpaEventoh::getIdeventodemonst($nrevento, $dtmescompetencia);
                self::insereBancoIteCalcFolha($ideventodemonst, $nrevento, $nrcalcfolha, $dtmescompetencia, $nrorg, $liquido, true, $nrlotevinc);
            }
        }
        if(!empty($vales)) {
            foreach($vales as $vale){
                $ideventodemonst = FpaEventoh::getIdeventodemonst(Fpatipoassoceve::ASSOC_VALE_TRANSP_RECEB, $dtmescompetencia);
                self::insereBancoIteCalcFolha($ideventodemonst, Fpatipoassoceve::ASSOC_VALE_TRANSP_RECEB, $nrcalcfolha, $dtmescompetencia, $nrorg, $vale, false, $nrlotevinc);
            }
        }
        Registry::getEntityManager()->flush();

    }

    /**
     * Função local responsável por por montar filtro de estrutura
     */
    public static function buildWhereEstrutura(&$nrestrutsind, &$nrestrutlegal, $nrestrutgeren){
        
        $condEstrutura = "";
        
        if(is_array($nrestrutsind)) $nrestrutsind = implode(",", $nrestrutsind);

        if(is_array($nrestrutlegal)) $nrestrutlegal = implode(",", $nrestrutlegal);
            
        $condEstrutura = " AND NRVINCULOM IN (SELECT VH.NRVINCULOM
                         FROM GPE_VINCULOH VH
                        WHERE VH.NRORG = :P_NRORG
                          AND ( INSTR(','||:P_NRESTRUTSIND||',', ','||VH.NRESTRUTSIND||',' ) > 0 OR :P_NRESTRUTSIND IS NULL )
                          AND ( INSTR(','||:P_NRESTRUTLEGAL||',', ','||VH.NRESTRUTLEGAL||',' ) > 0 OR :P_NRESTRUTLEGAL IS NULL )
                          AND (VH.NRESTRUTGEREN = :P_NRESTRUTGEREN OR :P_NRESTRUTGEREN IS NULL)
                          AND (INSTR(:P_LISTA_NRVINCULOM, '_'||VH.NRVINCULOM||'_') > 0 OR :P_LISTA_NRVINCULOM = 'A')
                          AND (VH.NRVINCULOM IN (SELECT NRVINCULOM FROM FPA_ITEMLOTEVINC WHERE NRORG = :P_NRORG AND INSTR(','||:P_LISTA_LOTE_VINCULOS||',', ','||NRLOTEVINC||',') > 0) OR :P_LISTA_LOTE_VINCULOS IS NULL)
                          AND ( INSTR(','
                                        || :P_NRESTRUTLEGALOPER
                                        || ',', ','
                                                || VH.NRESTRUTLEGAL
                                                || ',') > 0
                                  OR :P_NRESTRUTLEGALOPER IS NULL )
                            AND ( INSTR(','
                                        || :P_NRESTRUTGERENOPER
                                        || ',', ','
                                                || VH.NRESTRUTGEREN
                                                || ',') > 0
                                  OR :P_NRESTRUTGERENOPER IS NULL )
                          AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                    WHERE NRORG = VH.NRORG
                                                      AND NRVINCULOM = VH.NRVINCULOM
                                                      AND DTMESCOMPETENC <= :P_DTMESCOMPETENC))";

        return $condEstrutura;
    }
    
    /**
     * Função local responsável por por montar filtro de dt admissao
     */
    public static function buildWhereDtAdmissao(&$dtiniadmissao, &$dtfimadmissao){
        $condAdmissao = "";
        if(!empty($dtiniadmissao) && !empty($dtfimadmissao)){
            $condAdmissao = " AND NRVINCULOM IN (SELECT NRVINCULOM FROM GPE_VINCULOM WHERE NRORG = :P_NRORG AND DTADMISSAOVINC >= :P_DTINIADMISSAO AND DTADMISSAOVINC <= :P_DTFIMADMISSAO)";
        } else if(empty($dtiniadmissao) && !empty($dtfimadmissao)){
            $condAdmissao = " AND NRVINCULOM IN (SELECT NRVINCULOM FROM GPE_VINCULOM WHERE NRORG = :P_NRORG AND DTADMISSAOVINC <= :P_DTFIMADMISSAO)";
        } else if(!empty($dtiniadmissao) && empty($dtfimadmissao)){
            $condAdmissao = " AND NRVINCULOM IN (SELECT NRVINCULOM FROM GPE_VINCULOM WHERE NRORG = :P_NRORG AND DTADMISSAOVINC >= :P_DTINIADMISSAO)";
        }
        
        return $condAdmissao;
    }
    
    /**
     * Função local responsável por por montar filtro de estrutura
     */
    public static function buildWhereTpLinha($tipodelinha){
        
        $wheretipolinha = "";
        
        if($tipodelinha) {
            $wheretipolinha = "AND NRVINCULOM IN
                                (SELECT NRVINCULOM
                                FROM VAB_HISTVALEVINC
                                WHERE VAB_HISTVALEVINC.NRORG         = :P_NRORG
                                AND VAB_HISTVALEVINC.NRTIPOLINHAVINC = ".$tipodelinha."
                                AND e.NRVINCULOM      = VAB_HISTVALEVINC.NRVINCULOM
                                AND VAB_HISTVALEVINC.DTMESCOMPAPVALE = TO_DATE(:P_DTOCORRENCIA, 'DD/MM/YYYY')
                                )";
        }         
        
        return $wheretipolinha;
    }

    /**
     * Insere um item de cálculo de folha com o evento desejado
     * @param $nrevento
     * @param $nrcalcfolha
     * @param $dtmescompetencia
     * @param $nrorg
     * @param $info
     * @throws \Exception
     */
    private static function insereBancoIteCalcFolha($ideventodemonst, $nrevento, $nrcalcfolha, $dtmescompetencia, $nrorg, $info, $liquido = false, $nrlotevinc = null)
    {
        try {
            if(!empty($nrlotevinc)){
                $nrlotevinc = explode(',', $nrlotevinc);
                
                foreach($nrlotevinc as $nrlotevinculo){
                    $vinculoEncontrado = FpaEventoh::getVinculoLoteDeVinculo(FPA::getNrorgTrabalho(), $nrlotevinculo, $info['NRVINCULOM']);
                    
                    if(!empty($vinculoEncontrado)){
                        $nrlotevinc = $nrlotevinculo;
                        continue;
                    }
                }
            }
            
            if (($liquido) or (isset($info['DTPGTOVALE']))) {
                $dtocorrencia = $info['DTPGTOVALE'] == '' ? $dtmescompetencia : $info['DTPGTOVALE'];
                $params = array(
                    'P_NRORG' => $nrorg,
                    'P_NRVINCULOM'     => $info['NRVINCULOM'],
                    'P_NRCALCULOFOLHA' => $nrcalcfolha,
                    'P_NREVENTOM'      => $nrevento,
                    'P_DTOCORRENCIA'   => DateUtil::getDataDeString($dtocorrencia, DateUtil::FORMATO_BRASILEIRO, true));
            } else {
                $params = array(
                    'P_NRORG' => $nrorg,
                    'P_NRVINCULOM'     => $info['NRVINCULOM'],
                    'P_NRCALCULOFOLHA' => $nrcalcfolha,
                    'P_NREVENTOM'      => $nrevento,
                    'P_DTOCORRENCIA'   => DateUtil::getDataDeString($dtmescompetencia, DateUtil::FORMATO_BRASILEIRO, true));
            }

            $where = "NRORG        = :P_NRORG
                AND NRVINCULOM     = :P_NRVINCULOM
                AND NREVENTOM      = :P_NREVENTOM
                AND NRCALCULOFOLHA = :P_NRCALCULOFOLHA
                AND DTOCORRENCIA   = :P_DTOCORRENCIA";
            $uniqueitem = FpaItecalcfolha::getObjectByWhere($where, $params);
            if (!$uniqueitem) {
                $itemcalc = new FpaItecalcfolha();

                $itemcalc->setNritemcalcfolha(NovoCodigo::geraCodigo($nrorg, 'FPA_ITECALCFOLHA', 1, 21, true));
                $itemcalc->setCdoperinclusao(FPA::getCdoperador());
                $itemcalc->setCdoperultatu(FPA::getCdoperador());
                $itemcalc->setDtinclusao(DateUtil::getDataAtual());
                $itemcalc->setDtultatu(DateUtil::getDataAtual());
                $itemcalc->setIdevendemoncalc($ideventodemonst);
                $itemcalc->setIdrefevalorcalc('NORMAL');
                $itemcalc->setNrcalculofolha($nrcalcfolha);
                $itemcalc->setNreventom($nrevento);
                $itemcalc->setNrorg(FPA::getNrorgTrabalho());
                $itemcalc->setNrorginclusao(FPA::getNrorgTrabalho());
                $itemcalc->setNrorgultatu(FPA::getNrorgTrabalho());
                $itemcalc->setNrvinculom($info['NRVINCULOM']);
                $itemcalc->setQtocorproxperi(1);
                $itemcalc->setVreventofolha($info['VREVENTO']);
                $itemcalc->setVrreffolha($info['QTDIASVALE']);
                if (($liquido) or (isset($info['DTPGTOVALE']))) {
                    $dtocorrencia = $info['DTPGTOVALE'] == '' ? $dtmescompetencia : $info['DTPGTOVALE'];
                    $itemcalc->setDtocorrencia(DateUtil::getDataDeString($dtocorrencia, DateUtil::FORMATO_BRASILEIRO, true));
                } else {
                    $itemcalc->setDtocorrencia(DateUtil::getDataDeString($dtmescompetencia, DateUtil::FORMATO_BRASILEIRO, true));
                }

                if(!empty($nrlotevinc)){
                    $itemcalc->setNrlotevinc($nrlotevinc);
                } else {
                    $itemcalc->setNrlotevinc(null);
                }

                $itemcalc->persist();
            } else {
                $uniqueitem->setVreventofolha($info['VREVENTO']);
                $uniqueitem->setVrreffolha($info['QTDIASVALE']);
                if(!empty($nrlotevinc)){
                    $uniqueitem->setNrlotevinc($nrlotevinc);
                } else {
                    $uniqueitem->setNrlotevinc(null);
                }
                $uniqueitem->persist();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }



    /**
     * Busca se há líquidos para o lançamento
     *
     * @param int $nrcalculo
     * @param int $nrvinculom
     * @param String $dtmescompetencia
     * @param int $nrorg
     * @return bool
     */
    public static function buscaLiquido($nrcalculo, $nrvinculom, $dtmescompetencia, $nrorg)
    {
        $params = array(
            'P_NRORG'           => $nrorg,
            'P_NRCALCULOFOLHA'  => $nrcalculo,
            'P_DTOCORRENCIA'    => $dtmescompetencia,
            'P_NRVINCULOM'      => $nrvinculom
        );
        $where = "NREVENTOM IN (3030, 3040, 480) AND NRCALCULOFOLHA = :P_NRCALCULOFOLHA AND NRORG = :P_NRORG AND NRVINCULOM = :P_NRVINCULOM";
        $itemcalcs = FpaItecalcfolha::getObjectsByWhere($where, $params);

        if($itemcalcs){
            return true;
        }else{
            return false;
        }
    }

    /**
     * @param $lista_nrvinculos
     * @param $nrlinhatransp
     * @param $nrtarifa
     * @param $nrtipolinhavinc
     * @param $nrsentidolinha
     * @param $qtdiasvale
     * @param $dtpagtovale
     * @param $dtiniapurvale
     * @param $dtfimapurvale
     */
    public static function cadastroAutomatico($lista_nrvinculos, $nrlinhatransp, $nrtarifa, $nrtipolinhavinc, $nrsentidolinha, $qtdiasvale, $dtpagtovale, $dtiniapurvale, $dtfimapurvale)
    {
        try {
            $nrcalculofolha = FPA::determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_VALE_TRANSPORTE);

            foreach ($lista_nrvinculos as $nrvinculom) {
                $params = array(
                    'P_NRORG' => FPA::getNrorgTrabalho(),
                    'P_NRVINCULOM' => $nrvinculom,
                    'P_NRCALCULOFOLHA' => $nrcalculofolha,
                    'P_DTCOMPETENCIA' => FPA::getDtCompetencia(),
                    'P_DTPGTOVALE' => $dtpagtovale);

                $where = "NRORG           = :P_NRORG
                      AND NRVINCULOM      = :P_NRVINCULOM
                      AND NRCALCULOFOLHA  = :P_NRCALCULOFOLHA
                      AND DTMESCOMPAPVALE = :P_DTCOMPETENCIA
                      AND DTPGTOVALE      = :P_DTPGTOVALE";

                $uniqueitem = VabHistvalevinc::getObjectsByWhere($where, $params);

                if (!$uniqueitem) {
                    $histvalevinc = new VabHistvalevinc();

                    $histvalevinc->setNrcalculofolha($nrcalculofolha);
                    $histvalevinc->setDtmescompapvale(DateUtil::getDataDeString(FPA::getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true));
                    $histvalevinc->setNrvinculom($nrvinculom);
                    $histvalevinc->setNrlinhatransp($nrlinhatransp);
                    $histvalevinc->setNrtarifa($nrtarifa);
                    $histvalevinc->setNrtipolinhavinc($nrtipolinhavinc);
                    $histvalevinc->setNrsentidolinha($nrsentidolinha);
                    $histvalevinc->setQtdiasvale($qtdiasvale);
                    $histvalevinc->setDtpgtovale(DateUtil::getDataDeString($dtpagtovale, DateUtil::FORMATO_BRASILEIRO, true));
                    $histvalevinc->setDtiniapurvale(DateUtil::getDataDeString($dtiniapurvale, DateUtil::FORMATO_BRASILEIRO, true));
                    $histvalevinc->setDtfinapurvale(DateUtil::getDataDeString($dtfimapurvale, DateUtil::FORMATO_BRASILEIRO, true));
                    $histvalevinc->setQtapurvale(1);
                    $histvalevinc->setVrdeducaovale(0);
                    $histvalevinc->setNrorg(FPA::getNrorgTrabalho());
                    $histvalevinc->setNrorginclusao(FPA::getNrorgTrabalho());
                    $histvalevinc->setDtinclusao(DateUtil::getDataAtual());
                    $histvalevinc->setCdoperinclusao(FPA::getCdoperador());
                    $histvalevinc->setNrorgultatu(FPA::getNrorgTrabalho());
                    $histvalevinc->setDtultatu(DateUtil::getDataAtual());
                    $histvalevinc->setCdoperultatu(FPA::getCdoperador());

                    $nrhistvalevinc = NovoCodigo::geraCodigo(FPA::getNrorgTrabalho(),'VAB_HISTVALEVINC',1,10,true,false);
                    $histvalevinc->setNrhistvalevinc($nrhistvalevinc);

                    $histvalevinc->persist();
                }
                else {
                    throw new \Exception('Não cadastrado. Vínculo '.$vinculo['nrvinculom'].' já possui lançamento de vale.', 20);
                }
            }

            Registry::getEntityManager()->flush();

            throw new \Exception('Cadastro realizado com sucesso!',10);

        } catch (\Exception $e) {
            throw $e;
        }
    }
    public static function retornaVinculosTipoLinha($lista_nrvinculom, $tipodelinha, $nrorg, $dtmescompetencia){
        $params = array('P_NRORG'           => $nrorg,
            'P_NRTIPOLINHAVINC'  => $tipodelinha,
            'P_DTOCORRENCIA'    => $dtmescompetencia,
            'P_LISTA_NRVINCULOM'=> $lista_nrvinculom);
        $where = "  NRORG         = :P_NRORG
                    AND NRTIPOLINHAVINC = :P_NRTIPOLINHAVINC
                    AND (INSTR(:P_LISTA_NRVINCULOM, '_'
                      ||NRVINCULOM
                      ||'_')               > 0
                    OR :P_LISTA_NRVINCULOM = 'A')";
        $vinculos = VabHistvalevinc::getObjectsByWhere($where, $params);
        $lista_nrvinculom = '_';
        foreach($vinculos as $vinculo){
            $lista_nrvinculom .= $vinculo->getNrvinculom() . '_';
        }
        return $lista_nrvinculom;
    }

    /**
     * @param $lista_vinculos
     * @param $nrtipolinhavinc
     * @param $qtdiasacre
     * @param $qtdiasdesc
     * @throws \Exception
     */
    public static function alteraQtdeDiasVale($lista_vinculos, $nrtipolinhavinc, $qtdiasacre, $qtdiasdesc){
        try {

            Registry::getEntityManager()->beginTransaction();

            $nrorg = FPA::getNrorgTrabalho();
            $dtmescompetencia = FPA::getDtCompetencia();
            $nrcalculovale = FPA::determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_VALE_TRANSPORTE);

            $params = array(
                'P_NRORG' => $nrorg,
                'P_DTCOMPETENCIA' => $dtmescompetencia,
                'P_NRTIPOLINHAVINC' => $nrtipolinhavinc);

            $where = "NRORG = :P_NRORG AND
                      DTMESCOMPAPVALE = :P_DTCOMPETENCIA";

            if ($nrtipolinhavinc != ""){
                $where .= " AND NRTIPOLINHAVINC = $nrtipolinhavinc";
            }
            if ($lista_vinculos != "N"){
                $where .= " AND (INSTR('$lista_vinculos', '_'||NRVINCULOM||'_') > 0 OR '$lista_vinculos' = 'A')";
            }

            $vales = VabHistvalevinc::getObjectsByWhere($where, $params);

            /** @var VabHistvalevinc $vale */
            foreach ($vales as $vale) {

                $liquido = self::buscaLiquido($nrcalculovale, $vale->getNrvinculom(), $dtmescompetencia, $nrorg);

                if ($liquido) {
                    throw new \Exception("Existe(m) vínculo(s) calculado(s). Favor retroceder somente itens para depois alterar a quantidade de dias do vale.",20);
                    break;
                }

                $nrdias = $vale->getQtdiasvale();
                $nrdias = $nrdias + $qtdiasacre - $qtdiasdesc;
                $vale->setQtdiasvale($nrdias);
                $vale->prePersist();
            }

            Registry::getEntityManager()->flush();
            Registry::getEntityManager()->commit();
        } catch (\Exception $e) {
            Registry::getEntityManager()->rollback();
            throw $e;
        }
    }
    
    
    public static function realizaManutencaoAutomatica($nrorg, $dtcompetencia, $lista_lotevinculos, $lista_nrvinculom, $lista_nrlinhatransp, $lista_nrtipolinha, $dtpagtovaleini, $dtpagtovalefim, $dtiniapurvale, $dtfinapurvale, $qtapurvale, $qtdiasvale, $vrtarifamanutencao, $idajustadomanualmentemanu){
        $histvalevincs = \Folha\Repositories\FpaItecalcfolha::retornaHistoricosValeTransporte($nrorg, $dtcompetencia, $lista_lotevinculos, $lista_nrvinculom, $lista_nrlinhatransp, $lista_nrtipolinha, $dtpagtovaleini, $dtpagtovalefim, $dtiniapurvale, $dtfinapurvale);
        
        if($histvalevincs){
            try{
                foreach($histvalevincs as $histvalevinc){
                    
                    $params = array(
                        'P_NRORG' => $nrorg,
                        'P_DTCOMPETENCIA' => $dtcompetencia,
                        'P_NRHISTVALEVINC' => $histvalevinc['NRHISTVALEVINC']);
        
                    $where = "NRORG = :P_NRORG AND
                              DTMESCOMPAPVALE = :P_DTCOMPETENCIA AND
                              NRHISTVALEVINC = :P_NRHISTVALEVINC";
                    
                    $vales = VabHistvalevinc::getObjectsByWhere($where, $params);
                    
                    if($vales){
                        Registry::getEntityManager()->beginTransaction();
                        foreach($vales as $vale){

                            //Verifica se ja existe liquido.
                            $liquido = self::buscaLiquido($vale->getNrcalculofolha(), $vale->getNrvinculom(), $dtcompetencia, $nrorg);
                            
                            if ($liquido) {
                                throw new \Exception("Existe(m) vínculo(s) calculado(s). Favor retroceder somente itens para depois alterar a quantidade de dias do vale.",20);
                                break;
                            }
                            
                            if(($qtdiasvale == 0 || $qtdiasvale) && !empty($qtdiasvale)){
                                $vale->setQtdiasvale($qtdiasvale);
                            }
                            
                            if(($qtapurvale == 0 || $qtapurvale) && !empty($qtapurvale)){
                                $vale->setQtapurvale($qtapurvale);
                            }
                            
                            $vale->setIdajustemanual($idajustadomanualmentemanu);
                            
                            $vale->prePersist();
                        }
                        Registry::getEntityManager()->flush();
                        Registry::getEntityManager()->commit();
                    }
                }
            } catch (\Exception $e) {
                Registry::getEntityManager()->rollback();
                throw $e;
            }
        } else {
            //Não foram encontrados lançamentos para o filtro informado.
            throw new \Exception("Não foram encontrados lançamentos para o filtro informado.",20);
        }
    }
}